﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.ServiceModel.Activation;
using System.Text;
using Microsoft.ServiceModel.Web;
using Microsoft.ServiceModel.Web.SpecializedServices;

namespace RESTConsoleService
{
   class Program
   {
      static void Main(string[] args)
      {         
         CollectionServiceHost host = new CollectionServiceHost(typeof(CustomerService));
         host.EnableAutomaticHelpPage = false;
         host.AddServiceEndpoint(typeof (ICollectionService<Customer>), new WebHttpBinding(),
                                 "http://simon-hp:10100/RESTService/Customers").Behaviors.Add(new WebHttpBehavior2());         
         host.Open();

         Console.WriteLine("REST style service started.");
         Console.WriteLine("Press <enter> to exit.");
         Console.ReadLine();

         host.Close();   
      }
   }
   
   [ServiceBehavior(IncludeExceptionDetailInFaults = true, InstanceContextMode = InstanceContextMode.Single, ConcurrencyMode = ConcurrencyMode.Single)]   
   public class CustomerService : CollectionServiceBase<Customer>, ICollectionService<Customer>
   {
      readonly Dictionary<string, Customer> items = new Dictionary<string, Customer>();

      protected override IEnumerable<KeyValuePair<string, Customer>> OnGetItems()
      {
         return items;
      }

      protected override Customer OnGetItem(string id)
      {
         return items[id];
      }

      protected override Customer OnAddItem(Customer initialValue, out string id)
      {
         id = Guid.NewGuid().ToString();
         items.Add(id, initialValue);
         return initialValue;
      }

      protected override Customer OnUpdateItem(string id, Customer newValue)
      {
         Customer oldValue = OnGetItem(id);
         if (oldValue == null)
         {
            throw new WebProtocolException(HttpStatusCode.NotFound);
         }
         items[id] = newValue;
         return newValue;
      }

      protected override bool OnDeleteItem(string id)
      {
         Customer item = OnGetItem(id);
         if (item == null)
         {
            return false;
         }
         items.Remove(id);
         return true;
      }

   }

   [DataContract(Namespace = "")]
   public class Customer
   {
      [DataMember]
      public string FirstName { get; set; }
      [DataMember]
      public string LastName { get; set; }
      [DataMember]
      public DateTime BirthDate { get; set; }
      [DataMember]
      public int NumberOfWidgets { get; set; }
   }
}
